#define __KERNEL__


/* Saint Michael, Linux Kernel Module.
 * Verions: 0.03
 *
 * June 1, 2001
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

 
#include <linux/sys.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/smp.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <linux/reboot.h>
#include "StMichael_lkm.h"
#include "StMichael_string_util.h"

SM_INTEGRITY_RECORD  recorded_sys_call_table[NR_syscalls];

#ifdef USE_CHECKSUM

SM_INTEGRITY_RECORD recorded_dependency_table[NR_sm_dependencies];

int infractions = 0;

md5_byte_t dependency_md5[16];
md5_byte_t syscall_md5[16];

void sm_dependency_table_init( void )
{


  int i = 0;

  recorded_dependency_table[i++].orig_call = (void *) kmalloc;
  recorded_dependency_table[i++].orig_call = (void *) kfree;
#define __SM_PRINTK 2
  recorded_dependency_table[i++].orig_call = (void *) printk;
#define __SM_SCHEDULE 3
  recorded_dependency_table[i++].orig_call = (void *) schedule;
#define __SM_SYNC 4
  recorded_dependency_table[i++].orig_call = (void *) syscall_sync;
#define __SM_REBOOT 5 
  recorded_dependency_table[i++].orig_call = (void *) syscall_reboot;
  recorded_dependency_table[i++].orig_call = (void *) schedule_task;
  recorded_dependency_table[i++].orig_call = (void *) panic;
  recorded_dependency_table[i++].orig_call = (void *) machine_restart;
  recorded_dependency_table[i++].orig_call = (void *) machine_halt;
  recorded_dependency_table[i++].orig_call = (void *) machine_power_off;
  recorded_dependency_table[i++].orig_call = (void *) do_execve;
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0))
  recorded_dependency_table[i++].orig_call = (void *) kernel_read;
  recorded_dependency_table[i++].orig_call = (void *) open_exec;
#endif
  recorded_dependency_table[i++].orig_call = (void *) strnicmp;

  return;
}

void sm_integrity_check_checksum_init(SM_INTEGRITY_RECORD *record, void * target_function )
{

  md5_state_t state;
#ifdef USE_SHA1
  SHA1_CTX context;
#endif

  record->orig_call = target_function;
  
  md5_init(&state);
  md5_append(&state, (const md5_byte_t *) target_function,
             md5_sample_len );
  md5_finish(&state, record->recorded_md5_digest);

#ifdef USE_SHA1
  SHA1Init(&context);
  SHA1Update(&context,(unsigned char *) target_function, sha1_sample_len);
  SHA1Final(record->recorded_sha1_digest,&context);
#endif

  return;
  
}

#endif

void sm_integrity_check_init( void )
{

 register int i;

 
 // First, Syscalls.
 for ( i = 0; i < NR_syscalls; i++ )
 {

#ifdef USE_CHECKSUM

   sm_integrity_check_checksum_init( recorded_sys_call_table + i, sys_call_table[i] );

#else

   recorded_sys_call_table[i] = sys_call_table[i];

#endif

 }


// Then, if we are working with Checksums.. We do the Dependencies, and the Tables themselves.
    
#ifdef USE_CHECKSUM
 
 sm_dependency_table_init();
 for ( i = 0; i < NR_sm_dependencies; i++ )
    if( recorded_dependency_table[i].orig_call != NULL)
        { 
          sm_integrity_check_checksum_init( &recorded_dependency_table[i], 
                                recorded_dependency_table[i].orig_call );
        }


 {
   md5_state_t state;

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_sys_call_table,
            (sizeof(SM_INTEGRITY_RECORD) * NR_syscalls));
   md5_finish(&state, syscall_md5);


   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_dependency_table,
            (sizeof(SM_INTEGRITY_RECORD) * NR_sm_dependencies) );
   md5_finish(&state, dependency_md5);

 }

#endif

} 

#ifdef USE_CHECKSUM
int
sm_check_dependency_table()
{
  
  md5_state_t state;
  md5_byte_t digest[16];
  register int j;

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_dependency_table,
            (sizeof(SM_INTEGRITY_RECORD) * NR_sm_dependencies) );
   md5_finish(&state, digest);

    for (j = 0; j < 16; j++)
        if ( digest[j] != dependency_md5[j] )
                     return 1;
  
   return 0;
}

int
sm_check_sys_call_table()
{
  
  md5_state_t state;
  md5_byte_t digest[16];
  register int j;


   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_sys_call_table, 
          (sizeof(SM_INTEGRITY_RECORD) * NR_syscalls));

   md5_finish(&state, digest);

    for (j = 0; j < 16; j++)
        if ( digest[j] != syscall_md5[j] )
                     return 1;
  
   return 0;
}

/* 1 everything is ok
   0 its bad
 */
int
sm_check_specific_checksum(SM_INTEGRITY_RECORD *record)
{

  md5_state_t state;
  md5_byte_t md5_digest[16];
  unsigned char sha1_digest[20];
#ifdef USE_SHA1
  SHA1_CTX context;
#endif

  register int j;


   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) record->orig_call, 
          md5_sample_len);

   md5_finish(&state, md5_digest);

    for (j = 0; j < 16; j++)
        if ( md5_digest[j] != record->recorded_md5_digest[j] )
                     return 0;
#ifdef USE_SHA1
    SHA1Init(&context);
    SHA1Update(&context,(unsigned char *) record->orig_call, sha1_sample_len);
    SHA1Final(sha1_digest,&context);
   
    
    for (j = 0; j < 20; j++)
    { 
       if ( sha1_digest[j] != record->recorded_sha1_digest[j] )
                     return 0; 
    } 
#endif

   return 1;
}

void
sm_check_dependency_integrity ( void )
{


   register int i;

   if (sm_check_dependency_table())
           { 
#ifdef ROBINSON
             danger_will_robinson_danger();
#endif
                  return;
           }

   for (i = 0; i < NR_sm_dependencies; i++)
    {
       
       if (!sm_check_specific_checksum( recorded_dependency_table + i) )
             {
#ifdef ROBINSON
                  danger_will_robinson_danger();
#endif
                  return;
             } 
    }
       
        return;
            
}   
               

void danger_will_robinson_danger ( void )
{

// When in Fear, When In Doubt, Run Around, Scream and Shout!

    if (sm_check_specific_checksum( recorded_dependency_table + __SM_PRINTK))
                {
                  sm_print("(STMICHAEL) Non-Recoverable Attack Detected: The Kernel was modified directly.\n");

                  if (sm_check_specific_checksum( recorded_dependency_table + __SM_SYNC))
                  {
                      (*syscall_sync)();
                  }
                  if (sm_check_specific_checksum( recorded_dependency_table + __SM_SCHEDULE))
                   {
			schedule();
			schedule();
			schedule();
                   }
              
                }
               if (sm_check_specific_checksum( recorded_dependency_table + __SM_REBOOT))
                  {
                      sm_print("(STMICHAEL) Rebooting\n");
                      machine_restart(NULL);
                  }

                    // Sit, and Spin.
                    kernel_lock();
                    while(1) 
                         {
				;
                         }
}
               
#endif




void
sm_check_sys_call_integrity ( void  )
{
   register int i,j;


  /* 
     Verify that the syscall table is the same. 
     If its changed then respond 

   */

#ifdef USE_CHECKSUM 
   if (sm_check_sys_call_table())
     {
#ifdef ROBINSON
      danger_will_robinson_danger();
#endif
                  return;
     }

#endif 
	

   for (i = 0; i < NR_syscalls; i++)
#ifdef USE_CHECKSUM
    {

       if ( recorded_sys_call_table[i].orig_call != sys_call_table[i] )
#else
       if ( recorded_sys_call_table[i] != sys_call_table[i] )
#endif
          {

	    sm_print("(STMICHAEL) A Systemcall Table Change was detected on Moddelete. Possible Rootkit Present\n");
            sm_print("(STMICHAEL) Returning SysCall Table to Pristine State.\n");

#ifdef USE_CHECKSUM
                  sys_call_table[i] = recorded_sys_call_table[i].orig_call;
#else
            for ( j = 0; j < NR_syscalls; j++)
                  sys_call_table[j] = recorded_sys_call_table[j];

            break;
#endif
    
#ifdef USE_CHECKSUM
          }

       if ( !sm_check_specific_checksum( recorded_sys_call_table + i ) )
	    { 
#ifdef ROBINSON
                  danger_will_robinson_danger();
#endif
                  return;
               
            }
       
#endif           
            
      }   

  return;
}

