#define MODULE
#define __KERNEL__

/* Saint Michael, Linux Kernel Module.
 * Verions: 0.03
 *
 * June 1, 2001
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/sys.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <asm/segment.h>
#include <asm/unistd.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <asm/ptrace.h>
#include <asm/pgtable.h>
#include "StMichael_lkm.h"
#include "StMichael_string_util.h"
#include "StMichael_timer.h"

int
init_module(void)
{
    struct module *m, **v;

     
    /*
     * Our memory is not cleared for us. We need to init our hashes so that
     * each hash is like null. 
     */


    sm_print("--=={Loading %s\n ", VERSION);

#ifdef __SMP__
    rwlock_init(&sm_running);
#endif
    (void *) orig_init_module = sys_call_table[__NR_init_module]; 
    (void *) orig_delete_module = sys_call_table[__NR_delete_module];
    (void *) orig_exit = sys_call_table[__NR_exit];
    (void *) syscall_reboot = sys_call_table[__NR_reboot];
    (void *) syscall_sync = sys_call_table[__NR_sync];

    sys_call_table[__NR_init_module] = (void *) sm_init_module;
    sys_call_table[__NR_delete_module] = (void *) sm_delete_module; 
    sys_call_table[__NR_exit] = (void *) sm_exit; 


    sm_integrity_check_init();

#ifdef CLOAK
       // Now ya see me, 
       for ( m = THIS_MODULE; m->next != NULL; m = m->next );
       v = (struct module **) (++m);
       *v = (THIS_MODULE)->next;
       // Now ya don't.
#endif

// Activate the Timer...

      sm_timer_init();

    sm_print("--=={%s Successfully Loaded\n", VERSION);

    {

      // This function will self destruct in 5 seconds...

      unsigned long filler = 0xdeadbeef;
      unsigned long *p;

      for ( p = (unsigned long *) &m;
            p < (unsigned long *) &filler;
            p++)
		*p = filler;
 

    }

    return 0;
}

void
cleanup_module(void)
{
#if !defined(CLOAK)
    sys_call_table[__NR_init_module] = (void *) orig_init_module;
    sys_call_table[__NR_delete_module] = (void *) orig_delete_module; 
    sys_call_table[__NR_exit] = (void *) orig_exit; 
     
     /* 
       We should wait untill all calls are finished before ripping the 
       syscalls out from under the world ;)
     */

#ifdef __SMP__
    write_lock(&sm_running);
    write_unlock(&sm_running);
#endif // __SMP__
#endif

}
